package com.gic.enterprise.service.outer;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.enterprise.constant.Constants;
import com.gic.enterprise.dto.BillingOrderDTO;
import com.gic.enterprise.dto.DictDTO;
import com.gic.enterprise.dto.SmsPackageOrderItemDTO;
import com.gic.enterprise.entity.TabBillingCouponCard;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.service.BillingCouponCardService;
import com.gic.enterprise.service.BillingOrderApiService;
import com.gic.enterprise.service.BillingOrderService;
import com.gic.enterprise.service.DictApiService;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;

@Service("billingOrderApiService")
public class BillingOrderApiServcImpl implements BillingOrderApiService {

    private Logger logger = LogManager.getLogger(BillingOrderApiServcImpl.class);
    @Autowired
    private BillingOrderService billingOrderService;
    @Autowired
    private DictApiService dictApiService;
    @Autowired
    private BillingCouponCardService billingCouponCardService;


    @Override
    public ServiceResponse<Integer> saveSmsOrderBuyNow(BillingOrderDTO billingOrderDTO, SmsPackageOrderItemDTO smsPackageOrderItemDTO) {
        ServiceResponse<DictDTO> dictResponse = dictApiService.getDict(Constants.SMS_PACKAGE, smsPackageOrderItemDTO.getCode());
        if (!dictResponse.isSuccess()) {
            return ServiceResponse.failure(ErrorCode.NOTEXISTS.getCode(), "选择的套餐包不存在");
        }
        DictDTO dictDTO = dictResponse.getResult();
        smsPackageOrderItemDTO.setSmsAmount(Integer.valueOf(dictDTO.getKey()));
        smsPackageOrderItemDTO.setCost(Double.parseDouble(dictDTO.getContent()));
        billingOrderDTO.setItemJson(JSON.toJSONString(smsPackageOrderItemDTO));
        billingOrderDTO.setTotalFee(smsPackageOrderItemDTO.getSmsAmount() * smsPackageOrderItemDTO.getBuyCount() * smsPackageOrderItemDTO.getCost());
        // 判断优惠券是否存在
        TabBillingCouponCard couponCard = billingCouponCardService.getNotExpiredCouponCardById(billingOrderDTO.getCouponCardId());
        if (couponCard == null) {
            return ServiceResponse.failure(ErrorCode.NOTEXISTS.getCode(), "优惠券不存在或已过期");
        }
        Double content = couponCard.getContent();
        double totalFeePaid = billingOrderDTO.getTotalFee() - content;
        billingOrderDTO.setTotalFeePaid(totalFeePaid > 0 ? totalFeePaid : 0);


        Integer id = billingOrderService.saveOrderBuyNow(billingOrderDTO);
        return ServiceResponse.success(id);
    }

    @Override
    public ServiceResponse<Integer> updateOrderConfirmPay(BillingOrderDTO billingOrderDTO) {

        return null;
    }

    @Override
    public ServiceResponse<Page<BillingOrderDTO>> listOrder(Date startTime, Date endTime, Integer orderStatus, String search) {

        return null;
    }

    @Override
    public ServiceResponse<BillingOrderDTO> getOrderById(Integer orderId) {
        return null;
    }

    @Override
    public ServiceResponse<BillingOrderDTO> getOrderBySerialNumber(String serialNumber) {
        return null;
    }
}
