package com.gic.enterprise.service.outer;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.bizdict.api.dto.BizdictDTO;
import com.gic.bizdict.sdk.service.impl.RedisCacheBizdictService;
import com.gic.enterprise.constant.Constants;
import com.gic.enterprise.dto.DictDTO;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.service.DictApiService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

@Service("dictApiService")
public class DictApiServiceImpl implements DictApiService {

    @Autowired
    private RedisCacheBizdictService bizdictService;

    @Override
    public ServiceResponse<List<DictDTO>> listBusinessCategory() {
        List<BizdictDTO> bizDictList = bizdictService.treeByCategoryCode(Constants.BUSINESS_CATEGORY);
        List<DictDTO> list = bizDictList.stream().map(e -> {
            DictDTO dictDTO = new DictDTO();
            dictDTO.setKey(e.getName());
            dictDTO.setValue(e.getCode());
            return dictDTO;
        }).collect(Collectors.toList());
        return ServiceResponse.success(list);
    }

    @Override
    public ServiceResponse<DictDTO> getDict(String categoryCode, String code) {
        BizdictDTO bizdictDTO = bizdictService.getByCode(categoryCode, code);
        if (bizdictDTO != null) {
            DictDTO dictDTO = new DictDTO();
            dictDTO.setKey(bizdictDTO.getName());
            dictDTO.setValue(bizdictDTO.getCode());
            return ServiceResponse.success(dictDTO);
        } else {
            return ServiceResponse.failure(ErrorCode.NOTEXISTS.getCode(), ErrorCode.NOTEXISTS.getMsg());
        }
    }

    @Override
    public ServiceResponse<DictDTO> getDictOriginal(String categoryCode, String code) {
        BizdictDTO bizdictDTO = bizdictService.getByCode(categoryCode, code);
        if (bizdictDTO != null) {
            DictDTO dictDTO = new DictDTO();
            dictDTO.setKey(bizdictDTO.getName());
            dictDTO.setValue(bizdictDTO.getCode());
            dictDTO.setContent(bizdictDTO.getValueContent());
            return ServiceResponse.success(dictDTO);
        } else {
            return ServiceResponse.failure(ErrorCode.NOTEXISTS.getCode(), ErrorCode.NOTEXISTS.getMsg());
        }
    }
}
