package com.gic.enterprise.service.outer;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.GlobalInfo;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.constant.BrandCreateTypeEnum;
import com.gic.enterprise.constant.Constants;
import com.gic.enterprise.dto.DictDTO;
import com.gic.enterprise.dto.PlatformBrandDTO;
import com.gic.enterprise.dto.PlatformBrandRefDTO;
import com.gic.enterprise.entity.TabEnterprise;
import com.gic.enterprise.entity.TabPlatformBrand;
import com.gic.enterprise.entity.TabPlatformBrandRef;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.service.*;
import com.github.pagehelper.PageHelper;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Service("platformBrandApiService")
public class PlatformBrandApiServiceImpl implements PlatformBrandApiService {

    private Logger logger = LogManager.getLogger(PlatformBrandApiServiceImpl.class);

    @Autowired
    private PlatformBrandService platformBrandService;
    @Autowired
    private PlatformBrandRefService platformBrandRefService;
    @Autowired
    private DictApiService dictApiService;
    @Autowired
    private EnterpriseService enterpriseService;

    @Override
    public ServiceResponse<Integer> saveBrandFromGic(String categoryCode, String platformBrandName) {
        if (StringUtils.isBlank(platformBrandName) || StringUtils.isBlank(categoryCode)) {
            return ServiceResponse.failure(ErrorCode.MISS_PARAMETER.getCode(), ErrorCode.MISS_PARAMETER.getMsg());
        }
        TabPlatformBrand platformBrand = platformBrandService.getByBrandName(platformBrandName);
        // 如果平台品牌已存在， 直接返回； 这里先判断后执行，可能会出现并发问题，出问题了再加锁
        if (platformBrand != null) {
            return ServiceResponse.success(platformBrand.getPlatformBrandId());
        }
        PlatformBrandDTO platformBrandDTO = new PlatformBrandDTO();
        platformBrandDTO.setPlatformBrandCategoryCode(categoryCode);
        ServiceResponse<DictDTO> dictService = dictApiService.getDict(Constants.BUSINESS_CATEGORY, platformBrandDTO.getPlatformBrandCategoryCode());
        if (!dictService.isSuccess()) {
            return ServiceResponse.failure(dictService.getCode(), dictService.getMessage());
        }
        platformBrandDTO.setPlatformBrandCategoryName(dictService.getResult().getKey());
        platformBrandDTO.setCreateType(BrandCreateTypeEnum.GIC.getCode());
        platformBrandDTO.setPlatformBrandName(platformBrandName);
        Integer id = platformBrandService.save(platformBrandDTO);
        return ServiceResponse.success(id);
    }

    @Override
    public ServiceResponse<Integer> saveOrUpdateBrandFromOperation(PlatformBrandDTO platformBrandDTO) {
        if (StringUtils.isBlank(platformBrandDTO.getPlatformBrandName()) || StringUtils.isBlank(platformBrandDTO.getPlatformBrandCategoryCode())) {
            return ServiceResponse.failure(ErrorCode.MISS_PARAMETER.getCode(), ErrorCode.MISS_PARAMETER.getMsg());
        }
        TabPlatformBrand platformBrand = platformBrandService.getByBrandName(platformBrandDTO.getPlatformBrandName());
        // 如果平台品牌已存在， 直接返回； 这里先判断后执行，可能会出现并发问题，出问题了再加锁
        if (platformBrand != null) {
            return ServiceResponse.failure(ErrorCode.DATA_EXISTS.getCode(), platformBrand.getPlatformBrandId().toString());
        }
        ServiceResponse<DictDTO> dictService = dictApiService.getDict(Constants.BUSINESS_CATEGORY, platformBrandDTO.getPlatformBrandCategoryCode());
        if (!dictService.isSuccess()) {
            return ServiceResponse.failure(dictService.getCode(), dictService.getMessage());
        }
        platformBrandDTO.setPlatformBrandCategoryName(dictService.getResult().getKey());
        if (platformBrandDTO.getPlatformBrandId() == null) {
            // 新增
            platformBrandDTO.setCreateType(BrandCreateTypeEnum.PLATFORM.getCode());
            Integer id = platformBrandService.save(platformBrandDTO);
            platformBrandDTO.setPlatformBrandId(id);
        } else {
            // 编辑
            platformBrandService.update(platformBrandDTO);
        }
        return ServiceResponse.success(platformBrandDTO.getPlatformBrandId());
    }

    @Override
    public ServiceResponse<Integer> delBrandById(Integer platformBrandId) {
        Integer count = platformBrandRefService.countByPlatformBrandId(platformBrandId);
        if (count > 0) {
            return ServiceResponse.failure(ErrorCode.OPERATION_FAILED.getCode(), "该品牌下有关联的商户品牌，当前不允许删除！");
        }
        Integer line = platformBrandService.delBrandById(platformBrandId);
        if (line > 0) {
            return ServiceResponse.success(line);
        } else {
            return ServiceResponse.failure(ErrorCode.NOTEXISTS.getCode(), ErrorCode.NOTEXISTS.getMsg());
        }
    }

    @Override
    public ServiceResponse<Page<PlatformBrandDTO>> listPlatformBrand(PlatformBrandDTO platformBrandDTO, Integer pageNum, Integer pageSize) {
        com.github.pagehelper.Page<TabPlatformBrand> storeBrandList = platformBrandService.listPlatformBrand(platformBrandDTO, pageNum, pageSize);
        Page<PlatformBrandDTO> page = PageHelperUtils.changePageHelperToCurrentPage(storeBrandList, PlatformBrandDTO.class);
        if (CollectionUtils.isNotEmpty(storeBrandList)) {
            List<PlatformBrandDTO> platformBrandDTOList = page.getResult();
            List<Integer> platformBrandIdList = platformBrandDTOList.stream().map(PlatformBrandDTO::getPlatformBrandId).collect(Collectors.toList());
            Map<Integer, Integer> map = platformBrandRefService.countByPlatformBrandIds(platformBrandIdList);
            for (PlatformBrandDTO brandDTO : platformBrandDTOList) {
                brandDTO.setEnterpriseBrandCount(map.get(brandDTO.getPlatformBrandId()));
            }
        }
        return ServiceResponse.success(page);
    }

    @Override
    public ServiceResponse<List<PlatformBrandDTO>> listPlatformBrand(String brandName) {
        PlatformBrandDTO platformBrandDTO = new PlatformBrandDTO();
        platformBrandDTO.setPlatformBrandName(brandName);
        com.github.pagehelper.Page<TabPlatformBrand> storeBrandList = platformBrandService.listPlatformBrand(platformBrandDTO, 1, 10);
        List<PlatformBrandDTO> list = EntityUtil.changeEntityListNew(PlatformBrandDTO.class, storeBrandList);
        return ServiceResponse.success(list);
    }

    @Override
    public ServiceResponse<Integer> addRef(Integer enterpriseId, Integer enterpriseBrandId, String enterpriseBrandName, String enterpriseBrandCode, String categoryCode) {
        // 先判断后执行 可能并发
        TabPlatformBrand platformBrand = platformBrandService.getByBrandName(enterpriseBrandName);
        Integer platformBrandId;
        if (platformBrand == null) {
            ServiceResponse<Integer> serviceResponse = this.saveBrandFromGic(categoryCode, enterpriseBrandName);
            if (serviceResponse.isSuccess()) {
                platformBrandId = serviceResponse.getResult();
            } else {
                return serviceResponse;
            }
        } else {
            platformBrandId = platformBrand.getPlatformBrandId();
        }
        PlatformBrandRefDTO platformBrandRefDTO = new PlatformBrandRefDTO();
        platformBrandRefDTO.setPlatformBrandId(platformBrandId);
        platformBrandRefDTO.setEnterpriseId(enterpriseId);
        platformBrandRefDTO.setEnterpriseBrandId(enterpriseBrandId);
        platformBrandRefDTO.setEnterpriseBrandName(enterpriseBrandName);
        platformBrandRefDTO.setEnterpriseBrandCode(enterpriseBrandCode);
        Integer platformBrandRefId = platformBrandRefService.save(platformBrandRefDTO);
        logger.info("添加关联:{}", JSON.toJSONString(platformBrandRefDTO));
        return ServiceResponse.success(platformBrandRefId);
    }

    @Override
    public ServiceResponse<Integer> delRef(Integer enterpriseId, Integer enterpriseBrandId) {
        Integer line = platformBrandRefService.delRef(enterpriseId, enterpriseBrandId);
        logger.info("删除关联:[enterpriseId:{}, enterpriseBrandId:{}]", enterpriseId, enterpriseBrandId);
        return ServiceResponse.success(line);
    }

    @Override
    public ServiceResponse<Integer> delRef(Integer enterpriseId, List<Integer> enterpriseBrandIds) {
        Integer line = platformBrandRefService.delRef(enterpriseId, enterpriseBrandIds);
        logger.info("删除关联:[enterpriseId:{}, enterpriseBrandId:{}]", enterpriseId, enterpriseBrandIds);
        return ServiceResponse.success(line);
    }

    @Override
    public ServiceResponse<Integer> updateRef(Integer enterpriseId, Integer enterpriseBrandId, String enterpriseBrandName, String enterpriseBrandCode, String categoryCode) {
        // 先删除原来的
        platformBrandRefService.delRef(enterpriseId, enterpriseBrandId);
        return this.addRef(enterpriseId, enterpriseBrandId, enterpriseBrandName, enterpriseBrandCode, categoryCode);
    }

    @Override
    public ServiceResponse<PlatformBrandDTO> getByBrandName(String brandName) {
        TabPlatformBrand platformBrand = this.platformBrandService.getByBrandName(brandName);
        return ServiceResponse.success(EntityUtil.changeEntityByOrika(PlatformBrandDTO.class, platformBrand));
    }

    @Override
    public ServiceResponse<Page<PlatformBrandRefDTO>> listByPlatformBrandRef(String brandName, Integer pageNum, Integer pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        TabPlatformBrand brand = this.platformBrandService.getByBrandName(brandName);
        com.github.pagehelper.Page<TabPlatformBrandRef> brandRefPage = this.platformBrandRefService.listByPlatformBrandRef(brand == null ? null : brand.getPlatformBrandId());
        Page<PlatformBrandRefDTO> page = PageHelperUtils.changePageHelperToCurrentPage(brandRefPage, PlatformBrandRefDTO.class);
        if(CollectionUtils.isNotEmpty(page.getResult())) {
            for (PlatformBrandRefDTO ref : page.getResult()) {
                TabEnterprise enterprise = enterpriseService.getEnterpriseById(ref.getEnterpriseId());
                ref.setEnterpriseName(enterprise.getEnterpriseName());
                ref.setCompanyName(enterprise.getCompanyName());
                ref.setLogo(enterprise.getLogo());
            }
        }
        return ServiceResponse.success(page);
    }

    @Override
    public ServiceResponse<Integer> mergeBrandRef(String fromBrandId, String toBrandName) {
        if (StringUtils.isBlank(fromBrandId) || StringUtils.isBlank(toBrandName)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), ErrorCode.PARAMETER_ERROR.getMsg());
        }
        TabPlatformBrand platformBrand = this.platformBrandService.getByBrandName(toBrandName);
        if (platformBrand == null) {
            return ServiceResponse.failure(ErrorCode.NOTEXISTS.getCode(), "品牌名称不存在或已失效");
        }
        List<Integer> fromBrandIds = Stream.of(fromBrandId.split(GlobalInfo.FLAG_COMMA)).map(Integer::valueOf).collect(Collectors.toList());
        if (fromBrandIds.contains(platformBrand.getPlatformBrandId())) {
            return ServiceResponse.failure(ErrorCode.OPERATION_FAILED.getCode(), "目标品牌存在在源品牌中，请重新选择品牌");
        }
        Integer line = this.platformBrandRefService.updateRefs(fromBrandIds, platformBrand.getPlatformBrandId());
        if (line > 0) {
            platformBrandRefService.delBrandByIds(fromBrandIds);
            return ServiceResponse.success();
        } else {
            return ServiceResponse.failure(ErrorCode.NOTEXISTS.getCode(), ErrorCode.NOTEXISTS.getMsg());
        }
    }
}
